"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Nacl = require("tweetnacl");
const SHA3 = require("js-sha3");
const buffer_1 = require("buffer/"); // the trailing slash is important!
const hex_string_1 = require("./hex_string");
class AptosAccount {
    signingKey;
    accountAddress;
    authKeyCached;
    static fromAptosAccountPrivateKeyObject(obj) {
        return new AptosAccount(hex_string_1.HexString.ensure(obj.privateKeyHex).toUint8Array(), obj.address);
    }
    /** This class allows passing in an address, to handle account key rotation, where auth_key != public_key */
    constructor(privateKeyBytes, address) {
        if (privateKeyBytes) {
            this.signingKey = Nacl.sign.keyPair.fromSeed(privateKeyBytes.slice(0, 32));
        }
        else {
            this.signingKey = Nacl.sign.keyPair();
        }
        this.accountAddress = hex_string_1.HexString.ensure(address || this.authKey().hex());
    }
    /** Returns the address associated with the given account */
    address() {
        return this.accountAddress;
    }
    /** Returns the authKey for the associated account */
    authKey() {
        if (!this.authKeyCached) {
            const hash = SHA3.sha3_256.create();
            hash.update(buffer_1.Buffer.from(this.signingKey.publicKey));
            hash.update("\x00");
            this.authKeyCached = new hex_string_1.HexString(hash.hex());
        }
        return this.authKeyCached;
    }
    /** Returns the public key for the associated account */
    pubKey() {
        return hex_string_1.HexString.ensure(buffer_1.Buffer.from(this.signingKey.publicKey).toString("hex"));
    }
    signBuffer(buffer) {
        const signature = Nacl.sign(buffer, this.signingKey.secretKey);
        return hex_string_1.HexString.ensure(buffer_1.Buffer.from(signature).toString("hex").slice(0, 128));
    }
    signHexString(hexString) {
        const toSign = hex_string_1.HexString.ensure(hexString).toBuffer();
        return this.signBuffer(toSign);
    }
    toPrivateKeyObject() {
        return {
            address: this.address().hex(),
            publicKeyHex: this.pubKey().hex(),
            privateKeyHex: hex_string_1.HexString.fromUint8Array(this.signingKey.secretKey.slice(0, 32)).hex(),
        };
    }
}
exports.default = AptosAccount;
//# sourceMappingURL=aptos_account.js.map